/*******************************************************************************
  Main Source File

  Company:
    Microchip Technology Inc.

  File Name:
    main.c

  Summary:
    This file contains the "main" function for a project.

  Description:
    This file contains the "main" function for a project.  The
    "main" function calls the "SYS_Initialize" function to initialize the state
    machines of all modules in the system
 *******************************************************************************/

// *****************************************************************************
// *****************************************************************************
// Section: Included Files
// *****************************************************************************
// *****************************************************************************

#include <stddef.h>                     // Defines NULL
#include <stdbool.h>                    // Defines true
#include <stdlib.h>                     // Defines EXIT_FAILURE
#include <stdio.h>
#include <string.h>
#include "definitions.h"                // SYS function prototypes
#include "OLED128x64.h"

#define MCP9800_Addr    0x4d 

static volatile bool    isRTCTimerExpired = false;
volatile bool           bIsI2C_DONE = false ;
static uint8_t          uartTxBuffer[30] = {0};
unsigned char           I2C_ReadData[4];
unsigned char           I2C_WriteData[4];

const unsigned char  Class_String[]= "MU PLIB Lab3";
char                 OLED_Buffer[32];

#define         KXTJ3_ADDR      0x0e
char Xout ;
char Yout ;
char Zout ;

bool    KXTJ3_Process(unsigned char *i2cRdData);

// Lab3
static volatile bool    isDMATransferComplete = false;
#define         ACC_TRIG_VAL    12
bool            OutOfRange_Result ;
unsigned int    OutOfRangeCounter = 0 ;
unsigned int    BuzzerActiveCounter = 0 ;

static void rtcEventHandler (RTC_TIMER32_INT_MASK intCause, uintptr_t context)
{
    if (intCause & RTC_TIMER32_INT_MASK_COMPARE_MATCH)
    {
        /* RTC timer compare match. Here -----> Step #5 */
        isRTCTimerExpired = true;
    }
}
static void i2cHandler(uintptr_t context)
{
    if(SERCOM2_I2C_ErrorGet() == SERCOM_I2C_ERROR_NONE)
    {
         bIsI2C_DONE = true ;
    }
    else
    {
        //Error occurred during transfer.
    }
}

// Lab3 
static void dmaChannel0Handler (DMAC_TRANSFER_EVENT event, uintptr_t contextHandle)
{
    if (event == DMAC_TRANSFER_EVENT_COMPLETE)
    {
        isDMATransferComplete = true;
    }
}


// *****************************************************************************
// *****************************************************************************
// Section: Main Entry Point
// *****************************************************************************
// *****************************************************************************

int main ( void )
{
    /* Initialize all modules */
    SYS_Initialize ( NULL );
    
    DMAC_ChannelCallbackRegister(DMAC_CHANNEL_0, dmaChannel0Handler, 0);
    SERCOM2_I2C_CallbackRegister(i2cHandler, (uintptr_t)NULL);
    RTC_Timer32CallbackRegister(rtcEventHandler, 0);
    RTC_Timer32Start();

        while (!isRTCTimerExpired ) ;
        isRTCTimerExpired = false ;
    
        OLED_Init();
        OLED_CLS();
        OLED_Put8x16Str(0,0, Class_String ) ;
        
                // Write Control Reg.1 to Operation Mode
            bIsI2C_DONE = false ;
            I2C_WriteData[0] = 0x1b ;
            I2C_WriteData[1] = 0x80 ;
            SERCOM2_I2C_Write(KXTJ3_ADDR,I2C_WriteData,2);
                    while (bIsI2C_DONE == false)
                            {} ;             
        
    while ( true )
    {
        static bool status;
        /* Maintain state machines of all polled MPLAB Harmony modules. */
        SYS_Tasks ( );
        
        if (isRTCTimerExpired == true)
        {
            isRTCTimerExpired = false;           
            LED1_Toggle();
                     
            bIsI2C_DONE = false ;
            I2C_WriteData[0] = 0x06 ;
            SERCOM2_I2C_WriteRead(KXTJ3_ADDR,I2C_WriteData,1,I2C_ReadData, 6);
                    while (bIsI2C_DONE == false)
                            {} ;   
            OutOfRange_Result = KXTJ3_Process( &I2C_ReadData[0]) ;
            
            // Lab3
            if (OutOfRange_Result)
            {
                BUZZER_Set();   
                BuzzerActiveCounter = 2 ;
                sprintf((char*)uartTxBuffer,"You Lost Control %d Timers \r\n", OutOfRangeCounter++);
                status = DMAC_ChannelTransfer(DMAC_CHANNEL_0, uartTxBuffer, (const void *)&(SERCOM0_REGS->USART_INT.SERCOM_DATA), 
                         strlen((const char*)uartTxBuffer));
                if (status == false)
                {                /* Handle error condition */
                }        
            }
            
            if ( BuzzerActiveCounter)
            {
                BuzzerActiveCounter -= 1 ;
                if (BuzzerActiveCounter == 0 )
                    BUZZER_Clear();
            }
            
            
            
        }
  
    }

    /* Execution should not come here during normal operation */

    return ( EXIT_FAILURE );
}

bool    KXTJ3_Process(unsigned char *i2cRdData)
{
    bool    IsOutOfRange ;

            IsOutOfRange = false ;
            Xout = *(i2cRdData+1) ;   

            if (Xout < 128)
            {
                sprintf ((char*)OLED_Buffer,"X : +%-5d%c",Xout,0x00) ; 
                if (Xout > ACC_TRIG_VAL)      
                    IsOutOfRange = true ;               
            }
            else
            {
                Xout = 256-Xout ;
                sprintf ((char*)OLED_Buffer,"X : -%-5d%c",Xout,0x00) ; 
                if (Xout > ACC_TRIG_VAL)                                // Lab3   
                    IsOutOfRange = true ;         
            }
            OLED_Put8x16Str(0,2, (const unsigned char*)OLED_Buffer) ; 
            
                    Yout = *(i2cRdData+3);   
                    if (Yout < 128)
                    {
                        sprintf ((char*)OLED_Buffer,"Y : +%-5d%c",Yout,0x00) ;
                        if (Yout > ACC_TRIG_VAL)      
                            IsOutOfRange = true ;                 
                    }
                    else
                    {
                        Yout = 256-Yout ;
                        sprintf ((char*)OLED_Buffer,"Y : -%-5d%c",Yout,0x00) ;
                        if (Yout > ACC_TRIG_VAL)      
                            IsOutOfRange = true ;
                    }           
                    OLED_Put8x16Str(0,4, (const unsigned char*)OLED_Buffer) ; 
                
                        Zout = *(i2cRdData+5) ;              
                        if (Zout < 128)
                        {
                            sprintf ((char*)OLED_Buffer,"Z : +%-5d%c",Zout,0x00) ; 
                        }
                        else
                        {
                            Zout = 256-Zout ;
                            sprintf ((char*)OLED_Buffer,"Z : -%-5d%c",Zout,0x00) ;                 
                        }           
                        OLED_Put8x16Str(0,6, (const unsigned char*)OLED_Buffer) ;   
                        
                        return IsOutOfRange;
}

/*******************************************************************************
 End of File
*/